@@ -106,6 +106,8 @@ def get_ucode_problem(
     ucode svr get -d problems/ 44242
 
     """
+    CLog.error(f"This command is not supported anymore!")
+    return 
     ucode = UCode(env, domain=server)
     CLog.info(f"Getting problem `{problem_id}`...")
     dsa_problem = ucode.get_question(problem_id, download_testcases=True)
