@@ -1,3 +1,4 @@
+import codecs
 import copy
 import pickle
 
@@ -5,6 +6,7 @@ import requests
 from bs4 import BeautifulSoup
 from ucode.helpers.clog import CLog
 from ucode.models.problem import Problem, TestCase
+from ucode.oj.codeforces.aes import AESModeOfOperation
 from ucode.oj.codeforces.codeforces_helper import to_markdown
 from ucode.services.dsa.problem_service import ProblemService
 
@@ -12,8 +14,9 @@ from ucode.services.dsa.problem_service import ProblemService
 class Codeforces:
     def __init__(self):
         self.s = requests.session()
+        self._cookies = {}
         self._headers = {
-            'user-agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36',
+            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/94.0.4606.81 Safari/537.36',
             'origin': "https://codeforces.com",
             'referer': "https://codeforces.com/enter",
             'sec-fetch-dest': "empty",
@@ -22,9 +25,48 @@ class Codeforces:
             'x-requested-with': "XMLHttpRequest"
         }
         self.csrf = ''
+        self.rcpc = ''
         self.evercookie = "67ano0cpxyd6bygc7w"
         self.bfaa = "a60708ec87a45f5b510d7ab2b81a9044"
 
+    def gen_RCPC(self, url):
+        if self.rcpc:
+            return
+        r = self.s.get(url)
+        s = r.text
+        # print(r.text)
+        # , a, b, c
+        L = s.find("a=toNumbers") + 13
+        if L < 13:
+            return
+        R = s[L+20:].find('"') + L + 20
+        sa = s[L:R]
+        L = s.find("b=toNumbers") + 13
+        R = s[L + 20:].find('"') + L + 20
+        sb = s[L:R]
+        L = s.find("c=toNumbers") + 13
+        R = s[L + 20:].find('"') + L + 20
+        sc = s[L:R]
+        print("a, b, c:", sa, sb, sc)
+        a = codecs.decode(sa, 'hex_codec')
+        b = codecs.decode(sb, 'hex_codec')
+        c = codecs.decode(sc, 'hex_codec')
+
+        aes = AESModeOfOperation()
+        rcpc = aes.decrypt(c, None, 2, a, aes.aes.keySize["SIZE_128"], b)
+
+        rcpc_hex = ''
+        for c in rcpc:
+            code = ord(c)
+            h = hex(code)[2:]
+            if len(h) < 2:
+                h = '0' + h
+            rcpc_hex += h
+        print("RCPC:", rcpc_hex)
+        # self.rcpc = self._headers['cookie'] = f"RCPC={rcpc_hex}"
+        self._cookies = {"RCPC": rcpc_hex}
+        # self._headers['Set-Cookie'] = {"RCPC":rcpc_hex}
+
     def get_problem_list(self, page_index):
         url = f'https://codeforces.com/problemset/page/{page_index}'
 
@@ -100,8 +142,9 @@ class Codeforces:
         # print(problem.src_url)
         CLog.info("Getting problem from url: " + problem.src_url)
         url = problem.src_url
+        self.gen_RCPC(url)
         headers = copy.deepcopy(self._headers)
-        r = self.s.get(url, headers=headers)
+        r = self.s.get(url, headers=headers, allow_redirects=True, cookies=self._cookies)
         raw = r.text
         soup = BeautifulSoup(raw, 'html.parser')
         problem_statement = soup.select('div.problem-statement')[0]
@@ -208,7 +251,11 @@ class Codeforces:
         #     print(t)
         #     print('----')
 
-        submissions_cpp = self.get_submission_list(problem, 'cpp.g++11')
+        # create new session
+        self.s = requests.session()
+        self.get_csrf()
+
+        submissions_cpp = self.get_submission_list(problem, 'cpp.g++17')
         submissions_fpc = self.get_submission_list(problem, 'pas.fpc')
         submissions_py3 = self.get_submission_list(problem, 'python.3')
 
@@ -243,7 +290,6 @@ class Codeforces:
             return []
         frame_problem_index = problem.src_status_url[problem.src_status_url.rfind('/') + 1:]
         url = problem.src_status_url + "?order=BY_CONSUMED_TIME_ASC"
-
         headers = copy.deepcopy(self._headers)
 
         if not self.csrf:
@@ -266,14 +312,19 @@ class Codeforces:
             # '_tta': 248
         }
         # print(data)
-        r = self.s.post(url, headers=headers, data=data)
+        # print(headers)
+        CLog.info(f"Getting solution list from `{url}`")
+        r = self.s.post(url, headers=headers, data=data, allow_redirects=True, cookies=self._cookies)
         #
-        # print(r.status_code)
+        if r.status_code != 200:
+            r = self.s.get(url, headers=headers, data=data, allow_redirects=True, cookies=self._cookies)
+        print(r.status_code)
         # print(r.text)
         #
         # r = self.s.get(problem.src_status_url, headers=headers)
 
         raw = r.text
+        # print("status url:", url)
         # print(raw)
 
         soup = BeautifulSoup(raw, 'html.parser')
@@ -308,7 +359,7 @@ class Codeforces:
 
     def get_solution(self, submission, get_test_cases=False):
         # submission_url = submission['url']
-
+        print("get submission", submission)
         headers = copy.deepcopy(self._headers)
         # r = self.s.get(submission_url, headers=headers)
         # raw = r.text
@@ -316,12 +367,17 @@ class Codeforces:
         # print(raw)
         # source_code = soup.select('pre#program-source-text')[0].text
         # print(source_code)
-
-        r = self.s.post("https://codeforces.com/data/submitSource", headers=headers,
-                       data={
-                           'submissionId': submission['id'],
-                           'csrf_token': self.csrf
-                       })
+        data = {
+                   'submissionId': submission['id'],
+                   'csrf_token': self.csrf
+               }
+        url = "https://codeforces.com/data/submitSource"
+        r = self.s.post(url, headers=headers, data=data, allow_redirects=True)
+        print(self.s.cookies.get_dict())
+        print(headers)
+        print(data)
+        print("solutoin", r.text)
+        CLog.info(f"Getting solution detail from `{url}`")
         res = r.json()
         source_code = res['source']
 
@@ -332,8 +388,10 @@ class Codeforces:
         tesst_count = int(res['testCount'])
         testcases = []
         for t in range(tesst_count):
-            input_data = res['input#%s' % (t + 1)]
-            output_data = res['answer#%s' % (t + 1)]
+            input_data = res.get('input#%s' % (t + 1))
+            output_data = res.get('answer#%s' % (t + 1))
+            if not input_data or not output_data:
+                continue
             if input_data.endswith('...') or output_data.endswith('...'):  # uncompleted test data
                 continue
             testcases.append(TestCase(input_data.strip(), output_data.strip(), name=str(t + 1)))
@@ -389,10 +447,11 @@ class Codeforces:
 
     def get_csrf(self):
         headers = copy.deepcopy(self._headers)
-        r = self.s.get("https://codeforces.com/", headers=headers)
+        r = self.s.get("https://codeforces.com/", headers=headers, cookies=self._cookies)
         raw = r.text
         soup = BeautifulSoup(raw, 'html.parser')
         self.csrf = soup.select('span.csrf-token')[0]['data-csrf']
+        self._headers['x-csrf-token'] = self.csrf
         # print('CSRF:', self.csrf)
 
     def login(self, username, password):
@@ -529,13 +588,19 @@ class Codeforces:
 if __name__ == '__main__':
     cf = Codeforces()
     url = "https://codeforces.com/problemset/problem/268/A"
+    url = "https://codeforces.com/problemset/problem/1593/E"
+    # cf.gen_RCPC("e9ee4b03c1d0822987185d27bca23378", "188fafdbe0f87ef0fc2810d5b3e34705", "d047fb1e36224370a3f2cd7e9953c968")
+    # cf.gen_RCPC(url)
     dsa_problem = cf.get_problem_from_url(url)
+    #
+    # print(dsa_problem.name)
+    # for t in dsa_problem.testcases_sample:
+    #     print(t)
+    #
+    ProblemService.save(dsa_problem, r"D:\projects\ucode\ucode-cli\problems", overwrite=True)
+
 
-    print(dsa_problem.name)
-    for t in dsa_problem.testcases_sample:
-        print(t)
 
-    ProblemService.save(dsa_problem, "D:/projects/ucode/course-cpp/cpp101/lesson04-advanced-arrays", overwrite=True)
 
     # cf.login("thucnc", "15041985")
     # cf.submit(contest_id=4, submitted_problem_index="A", program_type_id=31, source="print('YES')")
