@@ -133,7 +133,9 @@ class TestcaseService:
             if file.lower().startswith("output/") \
                     or first_slash > 0 and file.lower()[first_slash + 1:].startswith("output/"):
                 has_output = True
-        if has_input and has_output:
+        if len(all_files) == 1:
+            format = "ucode"
+        elif has_input and has_output:
             format = "hackerrank"
             inputs = [f for f in all_files if (f.startswith("input/") or "/input/" in f) and not f.endswith("/")]
             test_files = []
@@ -540,10 +542,11 @@ if __name__ == "__main__":
     # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\cms\BANK.zip"
     # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\cms\cms_sample_nested.zip"
 
-    testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\bridge.zip"
+    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\bridge.zip"
+    testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\ucode\testcases.zip"
     testcases = TestcaseService.load_testcases(testcase_file)
     print(testcases)
-    print(len(testcases))
+    print("Count: ", len(testcases))
     print(testcases[0])
     print(testcases[1])
 
