@@ -41,6 +41,9 @@ class UCode:
         if env == "prod":
             self.api_base_url = "https://ucodeapis.com/api"
             self._headers['Referer'] = f'https://{domain}'
+        elif env == "stage":
+            self.api_base_url = "https://stage.ucodeapis.com/api"
+            self._headers['Referer'] = 'https://stage.ucode.vn'
         else:
             self.api_base_url = "https://dev.ucodeapis.com/api"
             self._headers['Referer'] = 'https://dev.ucode.vn'
@@ -107,6 +110,7 @@ class UCode:
 
     def upload_file(self, file_path, upload_folder):
         url = self.get_api_url(f"/general/upload-file?folder={upload_folder}")
+        print("url:", url, "file name:", file_path)
         # url = "https://httpbin.org/post"
         with open(file_path, 'rb') as f:
             file_name = os.path.basename(file_path)
@@ -115,6 +119,7 @@ class UCode:
             content_type = mimetypes.guess_type(file_path)[0]
             print("Content-Type:", content_type)
             files = {'file': (file_name, f, content_type)}
+
             response = self.s.post(url, files=files, headers=self._headers)
             # file_content = open(file_path, 'rb').read()
             # data = {'file': file_content}
@@ -157,7 +162,13 @@ class UCode:
 
         for i, test_data in enumerate(testcase_json):
             CLog.info(f"Downloading testcase #{i+1}...")
-            input_data = requests.get(test_data['input_url']).text
+            if not test_data['output_url']:
+                CLog.error(f"output_url of question #{question_id} is not valid")
+                raise ValueError(f"output_url of question #{question_id} is not valid")
+            if test_data['input_url']:
+                input_data = requests.get(test_data['input_url']).text
+            else:
+                input_data = ""
             output_data = requests.get(test_data['output_url']).text
             test = TestCase(input=input_data, output=output_data, sample=test_data['is_sample'])
             if test.sample:
@@ -215,13 +226,13 @@ class UCode:
 
             if ques['solutions']:
                 sol = json.loads(ques['solutions'])
-                sol = sol.get('text').strip()
-                if sol.startswith('```') and sol.endswith('```'):
-                    sol = "\n".join(sol.splitlines()[1:-1])
+                if sol and sol.get("text"):
+                    sol = sol.get('text').strip()
+                    if sol.startswith('```') and sol.endswith('```'):
+                        sol = "\n".join(sol.splitlines()[1:-1])
 
-                sol += "\n"
+                    sol += "\n"
 
-                if sol:
                     if "#include" in sol:
                         lang = "cpp"
                     elif "end." in sol:
@@ -234,6 +245,8 @@ class UCode:
                 self.download_testcases(question_id, problem)
 
             return problem
+        else:
+            return ques
 
     def get_question_from_quiz(self, quiz_id, save_to_folder=".", testcase_format="ucode"):
         url = self.get_api_url(f"/lesson-item/{quiz_id}/question")
@@ -250,12 +263,19 @@ class UCode:
         # questions = []
         save_to_folder = os.path.join(save_to_folder, f"quiz_{quiz_id}")
         os.makedirs(save_to_folder, exist_ok=True)
+
         for question_json in questions_json:
             ques_id = question_json['id']
-            CLog.info(f"Start downloading problem #{ques_id}")
-            problem = self.get_question(ques_id, download_testcases=True)
-            ProblemService.save(problem, save_to_folder, testcase_format=testcase_format)
-            # questions.append(question)
+            if question_json['type'] == "code":
+                CLog.info(f"Start downloading problem #{ques_id}")
+                problem = self.get_question(ques_id, download_testcases=True)
+                ProblemService.save(problem, save_to_folder, testcase_format=testcase_format)
+                # questions.append(question)
+            else:
+                question_json.update(self.get_question(ques_id, download_testcases=False))
+
+        with open(os.path.join(save_to_folder, f"quiz_{quiz_id}.json"), 'w', encoding="utf-8") as f:
+            f.write(json.dumps(questions_json, indent=2, ensure_ascii=False))
 
     def add_question_to_quiz(self, question_ids: List[int], quiz_id: int):
         url = self.get_api_url(f"/lesson-item/{quiz_id}/add-question")
@@ -304,8 +324,8 @@ class UCode:
         else:
             CLog.error("Cannot add questions to quiz:" + json.dumps(res))
 
-    def create_problem(self, problem: Union[Question, Problem, str],
-                       score=10, xp=10, lang='vi', statement_format="markdown", question_type='code',
+    def create_problem(self, problem: Union[Question, Problem, dict, str],
+                       score=10, xp=10, lang=None, statement_format="markdown", question_type='code',
                        status="published", visibility="unlisted", request_approval=False):
         """
         :param problem:
@@ -321,7 +341,10 @@ class UCode:
 
         if isinstance(problem, Problem):
             self.upload_and_replace_problem_resource(problem)
-
+            if not lang:
+                lang = problem.statement_language
+            if not lang:
+                lang = "vi"
             ucoin = xp
             if ucoin < 0:
                 # ucoin = int(round(50 * problem.difficulty**1.5, -2))
@@ -370,7 +393,12 @@ class UCode:
             # if problem.solution:
             #     data["solution"] = problem.solution
         elif isinstance(problem, Question):
+            if not lang:
+                lang = problem.statement_language
+            if not lang:
+                lang = "vi"
             question: Question = problem
+            print(f"statement language: {lang}, {problem.statement_language}")
             data = {
                 "name": question.src_name,
                 "type": question.type.value or question_type,
@@ -381,7 +409,7 @@ class UCode:
                 "status": status,
                 "visibility": visibility,
                 "ucoin": xp,
-                "constraints": "base_question" if question.base_question else
+                "source_detail": "base_question" if question.base_question else
                                                ("sub_question" if question.sub_question else "")
             }
 
@@ -426,8 +454,10 @@ class UCode:
 
             if question.answer is not None:
                 data['answers'] = question.answer
-            if question.type == QuestionType.SHORT_ANSWER:
-                print("data:", data)
+            # if question.type == QuestionType.SHORT_ANSWER:
+            #     print("data:", data)
+        elif isinstance(problem, dict):
+            data = problem
         else:
             raise Exception(f"Unsupported problem type {type(problem)}")
 
@@ -477,7 +507,7 @@ class UCode:
 
                     response = self.s.post(url, json=data, headers=self._headers)
                     print("status code", response.status_code)
-                    # print(response.text)
+                    print(response.text)
                     res = response.json()
                     if res['success']:
                         tran_question_id = res['data']['id']
@@ -487,7 +517,7 @@ class UCode:
 
         return question_id
 
-    def create_problems(self, problems: List[Union[Question, Problem, str]], score=10, xp=100, lang="vi", quiz_id=None):
+    def create_problems(self, problems: List[Union[Question, Problem, str]], score=10, xp=10, lang=None, quiz_id=None):
         question_ids = []
         for problem in problems:
             q_id = self.create_problem(problem=problem, score=score, xp=xp, lang=lang)
@@ -498,6 +528,30 @@ class UCode:
 
         return question_ids
 
+    def upload_all_questions_to_quiz(self, quiz_folder, quiz_id, real_upload=False, use_v1=False):
+        if use_v1:
+            problems = ProblemService.read_all_problems_v1(quiz_folder, nested_folder=False)
+        else:
+            problems = ProblemService.read_all_problems(quiz_folder, nested_folder=False)
+
+        for i, (problem_folder, problem) in enumerate(problems):
+            if use_v1:
+                problem = ProblemService.load_v1(problem_folder, load_testcase=True, translations=['vi'])
+            else:
+                problem = ProblemService.load(problem_folder, load_testcase=True)
+            # problem = ProblemService.load_v1(problem_folder, load_testcase=True, translations=['vi'])
+            print(i + 1, problem.name, "testcases:", len(problem.testcases), ", sample test:",
+                  len(problem.testcases_sample))
+
+            # print("translations", problem.statement_language, list(problem.translations.keys())[0])
+            if real_upload:
+                prob_id = self.create_problem(problem=problem, xp=-1, score=100)
+                CLog.info(f"Created problem: {prob_id}")
+                self.add_question_to_quiz(quiz_id=quiz_id, question_ids=[prob_id])
+                print(f"Problem uploaded with id #{prob_id} and added to quiz {quiz_id}")
+
+        print("Total problems:", len(problems))
+
     def upload_testcases(self, problem_id, testcases: List[TestCase], score=10, sample_score=0):
         url = self.get_api_url(f"/question/{problem_id}/add-testcases")
         testcase_data = []
@@ -563,8 +617,19 @@ class UCode:
 
         return api_url, token
 
-    def create_chapter(self, course_id, chapter_name, slug=None, parent_id=None,
-                       status="draft", _type="chapter", is_free=True):
+    def create_chapter_or_lesson(self, course_id, chapter_name, slug=None, parent_id=None,
+                                 status="draft", _type="chapter", is_free=True):
+        """
+
+        @param course_id:
+        @param chapter_name:
+        @param slug:
+        @param parent_id:
+        @param status: published / draft
+        @param _type: chapter / lesson
+        @param is_free:
+        @return: newly created chapter id
+        """
         if not slug:
             slug = make_slug(chapter_name)
 
@@ -573,7 +638,6 @@ class UCode:
             "item_type": _type,
             "name": chapter_name,
             "is_preview": False,
-            # "content_type": "video",
             "is_free": is_free,
             "slug": slug,
             "status": status
@@ -592,11 +656,13 @@ class UCode:
             raise Exception("Cannot create chapter:" + json.dumps(res))
 
     def create_lesson_item(self, course_id, chapter_id, lesson_name, slug=None,
-                           desciption="", content="",
+                           description="", content="", short_description="",
                            type="video", video_url="",
                            file_url="",
                            file_type="pdf",
-                           status="published", is_free=True, visibility="unlisted", ucoin=100):
+                           status="published", is_free=True, visibility="unlisted", ucoin=100,
+                           content_format="markdown",
+                           quiz_type="submit_single_question"):
         if not slug:
             slug = make_slug(lesson_name)
 
@@ -625,16 +691,17 @@ class UCode:
         lesson_item_id = res['data']['lesson_item_id']
         print("lesson_item_id:", lesson_item_id)
         # if type == "video":
-        if video_url or content or desciption:
+        if video_url or content or description:
             url = self.get_api_url(f"/lesson-item/{lesson_item_id}")
             data = {
-                "description": desciption,
+                "description": description,
+                "short_description": short_description,
                 "video_url": video_url,
                 "file_url": file_url,
                 "file_type": file_type,
                 "content": content,
                 "name": lesson_name,
-                "content_format": "markdown",
+                "content_format": content_format,
                 "slug": slug,
                 "is_free": is_free,
                 "is_preview": False,
@@ -643,6 +710,9 @@ class UCode:
                 "status": status,
             }
 
+            if type == "quiz":
+                data['quiz_type'] = quiz_type
+
             response = self.s.put(url, json=data, headers=self._headers)
             print(response.status_code)
             # print(response.text)
@@ -698,6 +768,97 @@ class UCode:
         lesson_item = data_json['data']
         return lesson_item
 
+    def create_course_curriculum(self, course_id, curriculum_file, create_quizes=False):
+        with open(curriculum_file, encoding="utf-8") as f:
+            curriculum = json.load(f)
+        print("curriculum:\n", json.dumps(curriculum))
+        folder = os.path.split(curriculum_file)[0]
+        for chapter in curriculum:
+            chapter['is_free'] = False
+            chapter['status'] = "published"
+
+            new_chapter_id = self.create_chapter_or_lesson(
+                course_id=course_id, chapter_name=chapter['name'],
+                status=chapter['status'], _type=chapter["item_type"], is_free=chapter['is_free'])
+
+            items = chapter['items']
+            for item in items:
+                if item['item_type'] == "lesson":
+                    item['is_free'] = False
+                    item['status'] = 'published'
+                    new_lesson_id = self.create_chapter_or_lesson(
+                        course_id=course_id, chapter_name=item['name'], parent_id=new_chapter_id,
+                        status=item['status'], _type=item["item_type"], is_free=item['is_free'])
+
+                    for lesson_item in item['items']:
+                        # lesson_item.update(self.get_lesson_item(lesson_item['lesson_item_id']))
+                        lesson_item['status'] = 'published'
+                        lesson_item['is_free'] = False
+                        new_lesson_item_id = self.create_lesson_item(
+                            course_id=course_id, chapter_id=new_lesson_id, lesson_name=lesson_item['name'],
+                            description=lesson_item['description'],
+                            short_description=lesson_item['short_description'],
+                            content=lesson_item['content'], type=lesson_item['content_type'],
+                            video_url=lesson_item['video_url'],
+                            file_url=lesson_item['file_url'],
+                            file_type=lesson_item['file_type'],
+                            is_free=lesson_item['is_free'],
+                            status=lesson_item['status'],
+                            visibility=lesson_item['visibility'],
+                            ucoin=lesson_item['ucoin']
+                        )
+                        if create_quizes and lesson_item['content_type'] == 'quiz':
+                            self.upload_question_from_local(lesson_item, folder, new_lesson_item_id)
+                else:  # lesson_item
+                    # item.update(self.get_lesson_item(item['lesson_item_id']))
+                    lesson_item = item
+                    lesson_item['status'] = 'published'
+                    lesson_item['is_free'] = False
+                    new_lesson_item_id = self.create_lesson_item(
+                        course_id=course_id, chapter_id=new_chapter_id, lesson_name=lesson_item['name'],
+                        description=lesson_item['description'],
+                        short_description=lesson_item['short_description'],
+                        content=lesson_item['content'], type=lesson_item['content_type'],
+                        video_url=lesson_item['video_url'],
+                        file_url=lesson_item['file_url'],
+                        file_type=lesson_item['file_type'],
+                        is_free=lesson_item['is_free'],
+                        status=lesson_item['status'],
+                        visibility=lesson_item['visibility'],
+                        ucoin=lesson_item['ucoin']
+                    )
+                    if create_quizes and lesson_item['content_type'] == 'quiz':
+                        self.upload_question_from_local(lesson_item, folder, new_lesson_item_id)
+            # break # test 1st chapter
+
+    def upload_question_from_local(self, lesson_item_json, base_folder, quiz_id):
+        local_quiz_id = lesson_item_json['lesson_item_id']
+        quiz_folder = os.path.join(base_folder, f"quiz_{local_quiz_id}")
+        with open(os.path.join(quiz_folder, f"quiz_{local_quiz_id}.json"), encoding="utf-8") as f:
+            quiz_json = json.load(f)
+        if quiz_json and quiz_json[0]['type'] == "code":
+            CLog.info(f"Uploading coding quiz from `{quiz_folder}`")
+            self.upload_all_questions_to_quiz(quiz_id=quiz_id,
+                                              quiz_folder=quiz_folder, real_upload=True)
+        else:
+            for question_json in quiz_json:
+                data = {}
+                for k in ["name", "type", "headline", "statement_language", "score",
+                          "status", "visibility", "ucoin",
+                          "source", "source_detail", "subject", "difficulty", "tags",
+                          "difficult_level", "slug", "display_type", "options",
+                          "option_display", "answers", "hint", "solutions",
+                          "input_desc", "output_desc", "constraints", "statement",
+                          "statement_format", "statement_media", "extra_info"]:
+                    if question_json[k]:
+                        data[k] = question_json[k]
+                    if not "name" in data:
+                        data['name'] = ""
+                ques_id = self.create_problem(problem=data)
+                CLog.info(f"Created problem: {ques_id}")
+                self.add_question_to_quiz(quiz_id=quiz_id, question_ids=[ques_id])
+                print(f"Problem uploaded with id #{ques_id} and added to quiz {quiz_id}")
+
 
 def create_chapters_mc1():
     ucode = UCode("dev")
@@ -705,7 +866,7 @@ def create_chapters_mc1():
     course_id = 7
     for w in range(1, 13):
         chapter_name = "Thử thách %02d" % w
-        chapter_id = ucode.create_chapter(course_id=course_id, chapter_name=chapter_name)
+        chapter_id = ucode.create_chapter_or_lesson(course_id=course_id, chapter_name=chapter_name)
 
 
 def create_problem(env="dev", domain="ucode.vn"):
@@ -750,53 +911,18 @@ def create_problem(env="dev", domain="ucode.vn"):
 
 
 def create_problems_to_lesson_ucode(env="dev", domain="ucode.vn"):
-    # base_folder = "D:\\projects\\ucode\\repl_backup\\ThucNguyen1\\py1-1-lesson-6-7-if"
-
-    # base_folder = "/home/thanh/Ucode/basic-problems/problem_set_01"
-    # base_folder = "D:\\projects\\ucode\\basic_problems\\hoc_vong_2\\secondary-school\\hinh_chu_nhat"
-    # base_folder = "D:\\projects\\ucode\\basic_problems\\math-coding-train\\secondary-school\\so_chinh_phuong"
-    # base_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\de_thi_hsg\\quan_ba_dinh_2020_2021_thcs_round2"
-    # base_folder = "D:\\projects\\ucode\\weekly-algorithm-problems\\competitions\\hsg-hn-2020_new"
-    # base_folder = "D:\\projects\\ucode\\repl_backup\\itteacher-k1\\quiz9_turtle"
-    # base_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\de_thi_hsg\\hanoi_2020_2021_thcs"
-    # base_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\de_thi_hsg\\vinhphuc_2020_2021_thcs"
-    # base_folder = "D:\\projects\\ucode\\repl_backup\\intro3"
-    # base_folder = "D:\\projects\\ucode\\basic_problems\\hour-of-code-round2"
-    # base_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\competitive_programing_topics\\maths\\fundamentals\\gcd_lcm_new"
-    # base_folder = "D:\\projects\\freecontest\\beginner_formatted\\Beginner Free Contest 27 (15-01-2021)"
-    # base_folder = "D:\\projects\\ucode\\competitive_programing_topics\\Maths\\Modulo & GCD - LCM\\Modular Arithmetic"
-    # base_folder = "D:\\projects\\freecontest\\formatted\\Free Contest 97"
-
-    # base_folder = r"D:\projects\ucode\weekly-algorithm-problems\competitions\teko-hcm"
-    # base_folder = r"D:\projects\freecontest\mock_tests\mock01"
-    # base_folder = r"D:\projects\ucode\competitive_programing_topics\tests\sample_tests\thcs_testTHT"
-    base_folder = r"D:\projects\ucode\dsa-problem-bank\lamcherry\2021-09\thcs_tests\test_03"
-
-    # base_folder = "D:\\projects\\ucode\\competitive_programing_topics\\Data_Structures\\Heap_PriorityQueue"
+    # base_folder = r"D:\projects\ucode\ucode-cli\problems\_save\c"
+    base_folder = r"D:\projects\ucode\basic-programming-problems\adhoc"
     problems = ProblemService.read_all_problems(base_folder, nested_folder=False)
     # problems = ProblemService.read_all_problems_v1(base_folder, nested_folder=False)
 
     # prob_folder = r"D:\projects\ucode\dsa-problem-bank\lamcherry\2021-09\thpt_tests\test_05\p04_bookshelves"
     # problems = [(prob_folder, ProblemService.load(prob_folder))]
 
-    lesson_id = 25478
+    lesson_id = 26434
     ucode = UCode(env=env, domain=domain)
-    #
-
-    for i, (problem_folder, problem) in enumerate(problems):
-        problem = ProblemService.load(problem_folder, load_testcase=True)
-        # problem = ProblemService.load_v1(problem_folder, load_testcase=True, translations=['vi'])
-        print(i + 1, problem.name, "testcases:", len(problem.testcases), ", sample test:", len(problem.testcases_sample))
-        # print("Solution:", json.dumps(problem.solutions))
-        # print("Editorial:", problem.editorial)
-        # print("Constraint: ", problem.constraints)
-
-        prob_id = ucode.create_problem(problem=problem, xp=-1, score=100)
-        CLog.info(f"Created problem: {prob_id}")
-        ucode.add_question_to_quiz(quiz_id=lesson_id, question_ids=[prob_id])
-        print(f"Problem uploaded with id #{prob_id} and added to quiz {lesson_id}")
-
-    print("Total problems:", len(problems))
+    # ucode.upload_all_questions_to_quiz(quiz_folder=base_folder, quiz_id=lesson_id, real_upload=False)
+    ucode.upload_all_questions_to_quiz(quiz_folder=base_folder, quiz_id=lesson_id,real_upload=True)
 
 
 def reupload_testcases():
@@ -812,16 +938,6 @@ def reupload_testcases():
     print(len(problem.testcases_sample))
 
 
-def test_login():
-    # ucode = UCode(env="prod", domain="teko.ugrow.vn")
-    # ucode = UCode(env="dev", domain="teko.ugrow.vn")
-    ucode = UCode(env="prod", domain="ucode.vn")
-    # ucode.login("teko@ugrow.vn", "123456")
-    # ucode.login("gthuc.nguyen@gmail.com", "ngoaho85")
-    # ucode.login("gthuc.nguyen@gmail.com", "12345678")
-    ucode.get_logged_in_user()
-
-
 def teko_bb():
     bbs = """Product	06/03/2020	BB#1: Women in Tech	Dương Thu Hương		https://docs.google.com/presentation/d/1j56LrIp-VCgzBGuNb84WHfVrCcpG5XJOBF7rdhR14hY/edit
 Product	10/12/2020	BB #23: Product Development Process	Hoàng Anh Duy	https://drive.google.com/file/d/1Lu8LzVpcl-REB_I5I1iEbGabIV_ihz9c/view	https://docs.google.com/document/d/13gnHsu9_5jU2cHAw940_inHXJ8YVbEXdPNBfNax_k0Y/edit
@@ -889,22 +1005,22 @@ UX/UI	12/11/2020	BB #19: How to use Teko Design System for PO an BA	Trần Thàn
         track, date, title, host, video_url, slide_url = bb.split("\t")
         print(track, host, slide_url)
         print(track_ids[track])
-        chapter_id = ucode.create_chapter(course_id=course_id, chapter_name=title, parent_id=track_ids[track],
-                                          status="published")
+        chapter_id = ucode.create_chapter_or_lesson(course_id=course_id, chapter_name=title, parent_id=track_ids[track],
+                                                    status="published")
         # chapter_id = 9482
         print("Chapter created:", chapter_id)
         lesson_id = ucode.create_lesson_item(course_id=course_id,
-                                 chapter_id=chapter_id,
-                                 lesson_name=title,
-                                 desciption=title,
-                                 type="lecture",
-                                 status = "published",
-                                 content=f"Date: {date}\n\nHost: {host}")
+                                             chapter_id=chapter_id,
+                                             lesson_name=title,
+                                             description=title,
+                                             type="lecture",
+                                             status = "published",
+                                             content=f"Date: {date}\n\nHost: {host}")
         print("Lesson info created:", lesson_id)
         lesson_id = ucode.create_lesson_item(course_id=course_id,
                                              chapter_id=chapter_id,
                                              lesson_name=title + " - Video",
-                                             desciption=title,
+                                             description=title,
                                              type="video",
                                              status="published",
                                              video_url=video_url,
@@ -913,7 +1029,7 @@ UX/UI	12/11/2020	BB #19: How to use Teko Design System for PO an BA	Trần Thàn
         lesson_id = ucode.create_lesson_item(course_id=course_id,
                                              chapter_id=chapter_id,
                                              lesson_name=title + " - Slide",
-                                             desciption=title,
+                                             description=title,
                                              type="file",
                                              file_url=slide_url,
                                              file_type="",
@@ -924,22 +1040,24 @@ UX/UI	12/11/2020	BB #19: How to use Teko Design System for PO an BA	Trần Thàn
 
 
 def test_get_question():
-    env, domain = "prod", "ucode.vn"
+    # env, domain = "prod", "ucode.vn"
+    env, domain = "prod", "tinhoctre.ucode.vn"
     # question_id = 44242
-    question_id = 97526
+    # question_id = 97526
+    question_id = 99888
 
     ucode = UCode(env=env, domain=domain)
 
     problem = ucode.get_question(question_id, download_testcases=True)
 
-    save_to = r"D:\projects\ucode\ucode-cli\problems\_save"
-    ProblemService.save(problem, save_to, overwrite=True, testcase_format="themis")
+    save_to = r"D:\projects\ucode\ucode-cli\problems\_save\c"
+    ProblemService.save(problem, save_to, overwrite=True, testcase_format="ucode")
 
 
 def test_get_questions_from_quiz():
     env, domain = "prod", "ucode.vn"
     ucode = UCode(env=env, domain=domain)
-    quiz_id = 3892
+    quiz_id = 75
     save_to = r"D:\projects\ucode\ucode-cli\problems\_save"
 
     ucode.get_question_from_quiz(quiz_id, save_to_folder=save_to)
@@ -949,14 +1067,14 @@ def add_to_daily_challenge():
     test_login()
     ucode = UCode(env="prod", domain="ucode.vn")
 
-    date = "2021-10-27"
-    folder = r"D:\projects\ucode\dsa-problem-bank\HTDung\2021-10\daily_challenges\challenge_002"
+    date = "2021-12-21"
+    folder = r"D:\projects\ucode\dsa-problem-bank\HTDung\2021-11\daily_challenges\challenge_027"
     problems = ProblemService.read_all_problems(folder, nested_folder=False)
 
     for i, (problem_folder, problem) in enumerate(problems):
         problem = ProblemService.load(problem_folder, load_testcase=True)
         print(i + 1, problem.name, "testcases:", len(problem.testcases), ", sample test:",
-              len(problem.testcases_sample))
+              len(problem.testcases_sample))te
 
         prob_id = ucode.create_problem(problem=problem, xp=-1, score=100)
         ucoin = [500,1000,2000][i]
@@ -970,25 +1088,134 @@ def test_get_course_curriculum():
     env, domain = "prod", "ucode.vn"
     ucode = UCode(env=env, domain=domain)
     # course_id = 822
-    course_id = 13
+    course_id = 822
     save_to = r"D:\projects\ucode\ucode-cli\problems\_save"
 
     ucode.get_course_curriculum(course_id, save_to_folder=save_to, get_quizes=True)
 
 
+def test_create_course_curriculum():
+    ucode = UCode(env="prod", domain="uschool.vn")
+    # ucode.login("gthuc.nguyen@gmail.com", "15041985")
+    # ucode.get_logged_in_user()
+    course_id = 830
+    curriculum_file = r"D:\projects\ucode\ucode-cli\problems\_courses\py101\course_822.json"
+    ucode.create_course_curriculum(course_id, curriculum_file, create_quizes=True)
+
+
+def test_create_question_from_json():
+    question_json = {
+        "id": 2,
+        "teacher_id": 1,
+        "teacher_name": None,
+        "teacher_avatar": None,
+        "name": "",
+        "type": "multiple_choice",
+        "headline": "",
+        "statement_language": "vi",
+        "programming_languages": None,
+        "code_stuff": None,
+        "resource_limit": None,
+        "root_question_id": None,
+        "score": 25,
+        "status": "published",
+        "compiler": "python",
+        "visibility": "unlisted",
+        "approval_status": None,
+        "approval_msg": None,
+        "ucoin": 25,
+        "languages": [
+          "vi"
+        ],
+        "source": None,
+        "source_detail": None,
+        "subject": "code",
+        "difficult_level": None,
+        "tags": [],
+        "difficulty": None,
+        "num_submitted_users": 1542,
+        "num_accepted_users": 1232,
+        "num_submission": 13009,
+        "num_accepted_submission": 1327,
+        "user_status": None,
+        "user_last_submitted_at": None,
+        "user_best_score": None,
+        "slug": "2",
+        "editable": True,
+        "enable_on_quiz": None,
+        "available_for_challenge": None,
+        "display_type": None,
+        "options": "[{\"text\": \"Ng\\u00f4n ng\\u1eef l\\u1eadp tr\\u00ecnh c\\u00f3 c\\u1ea5u tr\\u00fac (ng\\u1eef ph\\u00e1p) ch\\u1eb7t ch\\u1ebd\", \"text_type\": \"markdown\"}, {\"text\": \"Ng\\u00f4n ng\\u1eef l\\u1eadp tr\\u00ecnh \\u0111\\u01a1n gi\\u1ea3n h\\u01a1n r\\u1ea5t nhi\\u1ec1u so v\\u1edbi ng\\u00f4n ng\\u1eef t\\u1ef1 nhi\\u00ean\", \"text_type\": \"markdown\"}, {\"text\": \"Ng\\u00f4n ng\\u1eef l\\u1eadp tr\\u00ecnh l\\u00e0 c\\u00e1ch duy nh\\u1ea5t \\u0111\\u1ec3 giao ti\\u1ebfp v\\u1edbi m\\u00e1y t\\u00ednh\", \"text_type\": \"markdown\"}, {\"text\": \"Ng\\u00f4n ng\\u1eef l\\u1eadp tr\\u00ecnh d\\u00f9ng \\u0111\\u1ec3 t\\u1ea1o ra c\\u00e1c ch\\u01b0\\u01a1ng tr\\u00ednh m\\u00e1y t\\u00ednh\", \"text_type\": \"markdown\"}]",
+        "option_display": None,
+        "answers": "0,1,3",
+        "input_desc": "",
+        "output_desc": "",
+        "constraints": None,
+        "hint": "",
+        "solutions": None,
+        "statement": "Những điều nào sau đây là đúng khi nói về ngôn ngữ lập trình",
+        "statement_format": "markdown",
+        "statement_media": None,
+        "extra_info": None,
+        "max_submit_times": None,
+        "user_best_answer": None,
+        "user_answer_format": None,
+        "user_best_source_code": None,
+        "user_language_id": None,
+        "user_file_type": None,
+        "variant_question_id": None,
+        "sample_testcases": None,
+        "categories": []
+      }
+
+    data = {}
+    for k in ["name", "type", "headline", "statement_language", "score", "status", "visibility", "ucoin",
+              "source", "source_detail", "subject", "difficulty", "tags", "difficult_level",
+              "slug", "display_type", "options", "option_display", "answers", "hint", "solutions",
+              "input_desc", "output_desc", "constraints", "statement", "statement_format", "statement_media",
+              "extra_info"]:
+        if question_json[k]:
+            data[k] = question_json[k]
+        if not "name" in data:
+            data['name'] = ""
+
+    ucode = UCode(env="prod", domain="uschool.vn")
+    # ucode.login("gthuc.nguyen@gmail.com", "15041985")
+    # ucode.get_logged_in_user()
+    # course_id = 830
+    print("question data:", json.dumps(data))
+    ucode.create_problem(data)
+
+
+def test_login():
+    # ucode = UCode(env="prod", domain="teko.ugrow.vn")
+    # ucode = UCode(env="dev", domain="teko.ugrow.vn")
+    ucode = UCode(env="prod", domain="ucode.vn")
+    # ucode = UCode(env="prod", domain="tinhoctre.ucode.vn")
+    # ucode.login("teko@ugrow.vn", "123456")
+    # ucode.login("gthuc.nguyen@gmail.com", "15041985")
+    # ucode.login("gthuc.nguyen@gmail.com", "12345678")
+    ucode.get_logged_in_user()
+
+
 if __name__ == "__main__":
-    # test_login()
+    test_login()
     # teko_bb()
     # reupload_testcases()
 
+    # test_get_question()
+
     # create_problems_to_lesson_ucode("prod", domain="teko.ugrow.vn")
     # create_problems_to_lesson_ucode("prod")
 
-    test_get_question()
     # test_get_questions_from_quiz()
     # test_get_course_curriculum()
 
-    # add_to_daily_challenge()
+    # test_create_course_curriculum()
+
+    # test_create_question_from_json()
+
+    add_to_daily_challenge()
 
     # create_problem("dev")
     # create_problem("prod", domain="teko.ugrow.vn")
